# Generated by Django 5.1.5 on 2025-05-01 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0003_rename_ai_content_check_variable_profile_ai_content_check_provider_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='identity_id',
            field=models.CharField(default=None, max_length=46, null=True),
        ),
    ]
