# Generated by Django 5.2.4 on 2025-07-09 09:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0006_alter_profile_ai_create_attack_flow'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='profile',
            name='ai_summary_provider',
        ),
        migrations.AddField(
            model_name='profile',
            name='ai_extract_if_no_incidence',
            field=models.BooleanField(default=True),
        ),
    ]
