# Generated by Django 5.1.5 on 2025-02-20 11:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0002_profile_ai_content_check_variable'),
    ]

    operations = [
        migrations.RenameField(
            model_name='profile',
            old_name='ai_content_check_variable',
            new_name='ai_content_check_provider',
        ),
        migrations.AddField(
            model_name='profile',
            name='ai_create_attack_flow',
            field=models.BooleanField(default=True),
        ),
    ]
