# Generated by Django 5.1.5 on 2025-02-19 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='ai_content_check_variable',
            field=models.CharField(default=None, max_length=256, null=True),
        ),
    ]
